@@ -13,22 +13,22 @@ class Simulation(ABC):
     """ This class makes sure any subclasses have the necessary
         attributes to run a simulation.
     """
-    def __init__(self, name, output_path):
-        # set name and separator
-        self.name = name
-        self.separator = os.path.sep
-
-        # make the following paths
-        self.main_path = output_path + self.name + self.separator    # path to main simulation directory
-        self.images_path = self.main_path + name + "_images" + self.separator    # path to images output directory
-        self.values_path = self.main_path + name + "_values" + self.separator    # path to CSV output directory
+    def __init__(self):
+        # hold name, which will be overridden
+        self.name = None
 
         # hold the running number of agents and the step to begin at (updated by continuation mode)
         self.number_agents = 0
         self.current_step = 0
 
+        # make default arrays for the following agent values
+        self.locations = np.zeros((0, 3), dtype=float)
+        self.radii = np.ones(0, dtype=float)
+        self.hatching = np.zeros(0, dtype=bool)
+        self.removing = np.zeros(0, dtype=bool)
+
         # hold the names of the agent arrays and the names of any graphs (each agent is a node)
-        self.agent_array_names = list()
+        self.agent_array_names = ["locations", "radii", "hatching", "removing"]
         self.graph_names = list()
 
         # store the runtimes of methods with @record_time decorator
@@ -68,6 +68,12 @@ class Simulation(ABC):
         begin = self.number_agents
         self.number_agents += number
 
+        # extend the default arrays to the new number of agents
+        self.locations = np.concatenate((self.locations, np.zeros((number, 3), dtype=float)), axis=0)
+        self.radii = np.concatenate((self.radii, np.ones(number, dtype=float)))
+        self.hatching = np.concatenate((self.hatching, np.zeros(number, dtype=bool)))
+        self.removing = np.concatenate((self.removing, np.zeros(number, dtype=bool)))
+
         # if an agent type is passed
         if agent_type is not None:
             # make sure holder for types exists
@@ -97,12 +103,14 @@ class Simulation(ABC):
             # create instance variable and add array name to holder
             else:
                 self.__dict__[array_name] = override
-                self.agent_array_names.append(array_name)
+                if array_name not in self.agent_array_names:
+                    self.agent_array_names.append(array_name)
 
         # otherwise check if instance variable exists and try to make new array
         elif not hasattr(self, array_name):
             # add array name to holder
-            self.agent_array_names.append(array_name)
+            if array_name not in self.agent_array_names:
+                self.agent_array_names.append(array_name)
 
             # get the dimensions of the array
             if vector is None:
@@ -426,6 +434,55 @@ class Simulation(ABC):
         print("Step: " + str(self.current_step))
         print("Number of agents: " + str(self.number_agents))
 
+    def mark_to_hatch(self, index):
+        """ Mark the corresponding index of the array with True to
+            indicate that the agent should hatch a new agent.
+        """
+        self.hatching[index] = True
+
+    def mark_to_remove(self, index):
+        """ Mark the corresponding index of the array with True to
+            indicate that the agent should be removed.
+        """
+        self.removing[index] = True
+
+    @record_time
+    def update_populations(self):
+        """ Adds/removes agents to/from the simulation by adding/removing
+            indices from the cell arrays and any graphs.
+        """
+        # get indices of hatching/dying agents with Boolean mask
+        add_indices = np.arange(self.number_agents)[self.hatching]
+        remove_indices = np.arange(self.number_agents)[self.removing]
+
+        # count how many added/removed agents
+        num_added = len(add_indices)
+        num_removed = len(remove_indices)
+
+        # go through each agent array name
+        for name in self.agent_array_names:
+            # copy the indices of the agent array data for the hatching agents
+            copies = self.__dict__[name][add_indices]
+
+            # add/remove agent data to/from the arrays
+            self.__dict__[name] = np.concatenate((self.__dict__[name], copies), axis=0)
+            self.__dict__[name] = np.delete(self.__dict__[name], remove_indices, axis=0)
+
+        # go through each graph name
+        for graph_name in self.graph_names:
+            # add/remove vertices from the graph
+            self.__dict__[graph_name].add_vertices(num_added)
+            self.__dict__[graph_name].delete_vertices(remove_indices)
+
+        # change total number of agents and print info to terminal
+        self.number_agents += num_added - num_removed
+        print("\tAdded " + str(num_added) + " agents")
+        print("\tRemoved " + str(num_removed) + " agents")
+
+        # clear the hatching/removing arrays for the next step
+        self.hatching[:] = False
+        self.removing[:] = False
+
     def random_vector(self):
         """ Computes a random vector on the unit sphere centered
             at the origin.
@@ -452,6 +509,14 @@ class Simulation(ABC):
         for key in list(params.keys()):
             self.__dict__[key] = params[key]
 
+    def set_paths(self, output_dir):
+        """ Updates simulation output paths.
+        """
+        separator = os.path.sep
+        self.main_path = output_dir + self.name + separator  # path to main simulation directory
+        self.images_path = self.main_path + self.name + "_images" + separator  # path to images output directory
+        self.values_path = self.main_path + self.name + "_values" + separator  # path to CSV output directory
+
     @classmethod
     def start(cls, output_dir):
         """ Configures/runs the model based on the specified
@@ -463,9 +528,13 @@ class Simulation(ABC):
 
         # new simulation
         if mode == 0:
-            # check that new simulation can be made and create the Simulation object
+            # first check that new simulation can be made and create simulation output directory
             name = check_existing(name, output_dir, new_simulation=True)
-            sim = cls(name, output_dir)
+
+            # now make simulation instance, update name, and add paths
+            sim = cls()
+            sim.name = name
+            sim.set_paths(output_dir)
 
             # copy model files to simulation directory, ignoring __pycache__ files
             direc_path = sim.main_path + name + "_copy"
@@ -489,6 +558,9 @@ class Simulation(ABC):
                 with open(file_name, "rb") as file:
                     sim = pickle.load(file)
 
+                # update paths for the case the simulation is move to new folder
+                sim.set_paths(output_dir)
+
                 # iterate through all steps and create a video from any images
                 for sim.current_step in range(sim.current_step + 1, final_step + 1):
                     sim.step()
@@ -497,7 +569,9 @@ class Simulation(ABC):
             # images to video
             elif mode == 2:
                 # make object for video/path information and create video
-                sim = cls(name, output_dir)
+                sim = cls()
+                sim.name = name
+                sim.set_paths(output_dir)
                 sim.create_video()
 
             # zip simulation output
